/**
 * 孵化器基础数据相关逻辑
 * 作者: 李欣鸣
 * 孵化器相关逻辑 包括孵化器的增删改查
 * 
 */
import * as eccFormParamConfig from "../../config/eccFormParamConfig";
import { findEnterpriseCountByFuHuaQiUscc, groupFindEnterpriseCount } from "../../data/enterprise/enterprise";
import * as fuhuaqiData from "../../data/fuHuaQi/fuhuaqi";
import * as splitResultConfig from "../../config/splitResultConfig";
import * as configEnum from "../../config/enum";
import { checkChange, extractData } from "../../util/piecemeal";
import { BizError } from "../../util/bizError";
import { ERRORENUM } from "../../config/errorEnum";
import { eccFormParam } from "../../util/verificationParam";
import { changeEnumValue, eccEnumValue } from "../../util/verificationEnum";


/**
 * 主界面基础数据
 * 小程序端  首页的基础数据
 * 评分数据还没有
 * @param uscc 孵化器统一信用代码
 * @returns {} "incubatedAcreage", "acreageTotal", "acreagePersonalUse", "name", "lv", "institutionalNature"
 */
export async function homeData(uscc:string) {
    /** 这里中间件已经验证过uscc的可靠性了，不需要做多余的验证*/
    let dataBaseInfo = await fuhuaqiData.findFuHuaQiByUSCC(uscc);

    /**根据配置截取返回的字段 */
    let data:any = extractData(splitResultConfig.FuHuaQiHomeDataConfig, dataBaseInfo, false);
    data.enterpriseCount  = await findEnterpriseCountByFuHuaQiUscc(uscc) || 0;
    data.lv = changeEnumValue(configEnum.FUHUAQILV, dataBaseInfo.lv);
    data.institutionalNature = changeEnumValue(configEnum.INSTITUTIONALNATURE, dataBaseInfo.institutionalNature);

    return data;
}


/**
 * 孵化器详细数据
 * 小程序端  点击我的数据之后拉取的详细数据
 * @param uscc 孵化器统一信用代码
 * @returns data 基础数据字段
 */
export async function selectFuHuaQiBaseData(uscc:string) {
    /** 这里中间件已经验证过uscc的可靠性了，不需要做多余的验证*/
    let dataBaseInfo = await fuhuaqiData.findFuHuaQiByUSCC(uscc);

    /**截取返回 */
    let data:any = extractData(splitResultConfig.FuHuaQiBaseDataConfig, dataBaseInfo, false);
    data.enterpriseTotal =  await findEnterpriseCountByFuHuaQiUscc(uscc); //企业总数
    data.numOfEntities = data.enterpriseTotal - dataBaseInfo.virtualEnterpriseNum; //实体企业总数 = 虚拟企业总数 - 虚拟企业数量

    return data;
}


/**
 * 孵化器机构信息数据数据
 * 小程序端  点击机构信息之后拉取的详细数据
 * @param uscc 孵化器统一信用代码
 * @returns data 基础数据字段
 */
export async function selectOrganizationInfo(uscc:string) {
    /** 这里中间件已经验证过uscc的可靠性了，不需要做多余的验证*/
    let dataBaseInfo = await fuhuaqiData.findFuHuaQiByUSCC(uscc);
    
    /** 根据配置截取返回 */
    let data:any = extractData(splitResultConfig.OrganizationBaseDataConfig, dataBaseInfo, false);

    return data;
}


/**
 * 修改孵化器中我的数据信息
 * 小程序端
 * @param uscc 孵化器统一信用代码
 * @param param 表单内容
 * @returns {isSuccess:true/false};
 */
export async function updateFuHuaQiBaseData(uscc:string, param) {
    /**表单校验 */
    eccFormParam("更新孵化器基础数据", eccFormParamConfig.BaseParamUpdateConfig, param);
    if (param.hatchingGround && param.hatchingGround.length) {
        param.hatchingGround.forEach(hatchingGroundInfo => {
            eccFormParam("更新孵化器基础数据 经备案孵化场地", eccFormParamConfig.BaseParamUpdateHatchingGroundConfig, hatchingGroundInfo);
        });
    }

    let baseDataInfo = await fuhuaqiData.findFuHuaQiByUSCC(uscc);
    /**这里无法判断数组里面的内容是否有变化 所以 hatchingGround 直接赋值  */
    let changeList = checkChange(param, baseDataInfo);
    for (let i = 0; i < changeList.length; i++) {
        let key = changeList[i];
        baseDataInfo[key] = param[key];
    }
    baseDataInfo["hatchingGround"] = param.hatchingGround;

    await baseDataInfo.save();

    return {isSuccess:true};
}


/**
 * 更新孵化器机构信息数据
 * 小程序端
 * @param uscc 孵化器统一信用代码
 * @param param 表单内容
 * @returns {isSuccess:true/false};
 */
export async function updateOrganizationData(uscc:string, param) {
    /**校验表单内容 */
    eccFormParam("更新孵化器机构信息数据",  eccFormParamConfig.OrganizationParamUpdateConfig, param);
    if (param.foundingTeamType && param.foundingTeamType != configEnum.TEAM.无) {
        param.foundingTeam.forEach((subInfo, subIndex) => {
            eccFormParam(`更新孵化器机构信息数据 创始团队 ${subIndex}`,  eccFormParamConfig.OrganizationParamFoundingTeamUpdateConfig, subInfo);
            eccEnumValue(`更新孵化器机构信息数据 创始团队 ${subIndex}`, "团队类型", configEnum.TEAM, subInfo.type);
        }); 
    } else param.foundingTeam = [];
    if (param.operationModel && param.operationModel.length && param.operationModel.indexOf(configEnum.OPERATIONMODEL.其他) > 0 ) {
        if (!param.operationModelDes) throw new BizError(ERRORENUM.表单校验失败, `缺失运营模式描述`);
    }
    /**校验是否符合枚举范围 */
    if (param.lv) eccEnumValue("更新孵化器机构信息数据", " 孵化器级别 ", configEnum.FUHUAQILV, param.fuHuaQiInvestmentStyle);
    if (param.industry) eccEnumValue("更新孵化器机构信息数据", " 领域 ", configEnum.FUHUAINDUSTRY, param.fuHuaQiInvestmentStyle);
    if (param.institutionalNature) eccEnumValue("更新孵化器机构信息数据", " 机构性质 ", configEnum.INSTITUTIONALNATURE, param.fuHuaQiInvestmentStyle);
    if (param.operationModel && param.operationModel.length) eccEnumValue("更新孵化器机构信息数据", " 运营模式 ", configEnum.OPERATIONMODEL, param.fuHuaQiInvestmentStyle);
    
    let baseDataInfo = await fuhuaqiData.findFuHuaQiByUSCC(uscc);
    
    /**赋值内容  */
    let changeList = checkChange(param, baseDataInfo);
    for (let i = 0; i < changeList.length; i++) {
        let key = changeList[i];
        if (key == "operationName" || key == "uscc") continue;
        
        baseDataInfo[key] = param[key];
    }
    /**这里无法判断数组里面的内容是否有变化 所以 foundingTeam 直接赋值 */
    baseDataInfo["foundingTeam"] = param.foundingTeam;

    await baseDataInfo.save();

    return {isSuccess:true};
}


/**
 * 获取孵化器基本信息列表
 * 管理后台端
 * @param lv 孵化器等级
 * @param institutionalNature 机构性质
 * @param industry 领域
 * @param operationModel 运营模式
 * @param foundingTeam 创业团队
 * @param page 页数
 * @returns 
 */
export async function fuHuaQiBaseList(lv:number, institutionalNature:number, industry:number, operationModel:number, foundingTeam:number, page:number) {
    /**拼接查询条件 并且校验是否符合枚举范围  可优化 抽出一个方法统一校验 */
    let selectParam:any = {};
    if (lv) {
        eccEnumValue("管理后台查询孵化器列表", " 孵化器级别 ", configEnum.FUHUAQILV, lv);
        selectParam.lv = lv;
    }
    if (institutionalNature) {
        eccEnumValue("管理后台查询孵化器列表", " 机构性质 ", configEnum.INSTITUTIONALNATURE, institutionalNature);
        selectParam.institutionalNature = institutionalNature;
    }
    if (industry){
        eccEnumValue("管理后台查询孵化器列表", " 领域 ", configEnum.FUHUAINDUSTRY, industry);
        selectParam.industry = industry;
    }
    if (operationModel) {
        eccEnumValue("管理后台查询孵化器列表", " 运营模式 ", configEnum.OPERATIONMODEL, operationModel);
        selectParam.operationModel = operationModel;
    }
    if (foundingTeam) {
        eccEnumValue("管理后台查询孵化器列表", " 创业团队 ", configEnum.TEAM, foundingTeam);
        selectParam.foundingTeamType = foundingTeam;
    }

    let enterpriseMap = await groupFindEnterpriseCount();//计算孵化器拥有的企业总数

    let dataBaseList = await fuhuaqiData.findFuHuaQiListByPage(selectParam, (page-1)*10 );
    let count = await fuhuaqiData.findFuHuaQiCount(selectParam);

    /**拼接返回结果 */
    let dataList = [];
    dataBaseList.forEach( info => {
        /**截取所需字段 */
        let changeData:any = extractData(splitResultConfig.FuHuaQiBaseListConfig, info, true);

        const EnterpriseTotal = enterpriseMap[info.uscc] || 0; //企业总数
        let numOfEntities = EnterpriseTotal - (info.virtualEnterpriseNum || 0 ); //实体企业
        changeData.enterpriseTotal = EnterpriseTotal;
        changeData.numOfEntities = numOfEntities < 0 ? 0 : numOfEntities;


        /**枚举转换 */
        changeData.lv = changeEnumValue(configEnum.FUHUAQILV, changeData.lv);
        changeData.industry = changeEnumValue(configEnum.FUHUAINDUSTRY, changeData.industry);
        changeData.institutionalNature = changeEnumValue(configEnum.INSTITUTIONALNATURE, changeData.institutionalNature);
        changeData.operationModel = changeEnumValue(configEnum.OPERATIONMODEL, changeData.operationModel);
        
        changeData.foundingTeamStr = changeData.foundingTeamStr ? changeEnumValue(configEnum.TEAM, changeData.foundingTeamStr) :  "无";//创始团队文字
        let foundingTeam = [];
        if (changeData.foundingTeam && changeData.foundingTeam.length) {
            changeData.foundingTeam.forEach(item => {
                let {memberName,  memberAge,  memberSchool,  des} = item;
                let type = changeEnumValue(configEnum.TEAM, item.type);//人才类型
                foundingTeam.push({
                    memberName,  memberAge,  memberSchool,  des, type,
                    memberSex : changeEnumValue(configEnum.SEX, item.memberSex),//成员性别
                    memberDEGREE : changeEnumValue(configEnum.DEGREE, item.memberDEGREE),//成员最高学历
                });

            });
            changeData.foundingTeam = foundingTeam;
        }

        dataList.push(changeData);
    });

    return {count, dataList};
}



/**
 * 导出符合条件的孵化器基本信息数据 
  * @param lv 孵化器等级
 * @param institutionalNature 机构性质
 * @param industry 领域
 * @param operationModel 运营模式
 * @param foundingTeam 创业团队
 * @param type 导出类型 OUTPUTTYPE 枚举
 * @returns 
 */
export async function outPutFuHuaQiBaseData(lv:number, institutionalNature:number, industry:number, operationModel:number, foundingTeam:number, type:Number) {
    let selectParam:any = {};
    if (type == configEnum.OUTPUTTYPE.当前数据) {
        /**当选择了当前数据时 拼接查询条件 */
        let selectParam:any = {};
        if (lv) {
            eccEnumValue("管理后台查询孵化器列表", " 孵化器级别 ", configEnum.FUHUAQILV, lv);
            selectParam.lv = lv;
        }
        if (institutionalNature) {
            eccEnumValue("管理后台查询孵化器列表", " 机构性质 ", configEnum.INSTITUTIONALNATURE, institutionalNature);
            selectParam.institutionalNature = institutionalNature;
        }
        if (industry){
            eccEnumValue("管理后台查询孵化器列表", " 领域 ", configEnum.FUHUAINDUSTRY, industry);
            selectParam.industry = industry;
        }
        if (operationModel) {
            eccEnumValue("管理后台查询孵化器列表", " 运营模式 ", configEnum.OPERATIONMODEL, operationModel);
            selectParam.operationModel = operationModel;
        }
        if (foundingTeam) {
            eccEnumValue("管理后台查询孵化器列表", " 创业团队 ", configEnum.TEAM, foundingTeam);
            selectParam.foundingTeamType = foundingTeam;
        }
    }

    let enterpriseMap = await groupFindEnterpriseCount();//计算所有孵化器拥有的企业总数
    let dataBaseList = await fuhuaqiData.findFuHuaQiList(selectParam);

    /**拼接二维数组 */
    let keyList = [
        "name", "operationName", "uscc", "enterpriseTotal", "virtualEnterpriseNum", "numOfEntities", "logonTime", "incubatedAcreage",
        "acreageTotal", "acreagePersonalUse", "lv", "identificationTime", "industry", "institutionalNature",
        "liaison", "liaisonPhone", "personInCharge", "personInChargePhone", "operationModel", "foundingTeamType",
    ];
    let titleList = [
        "名称", "运营机构名称", "统一信用代码", "企业总数", "虚拟孵化", "实体孵化",
        "注册时间", "在孵面积", "孵化器总面积", "孵化器自用面积", "孵化器级别", "认定时间",
        "领域", "机构性质", "联系人", "联系电话", "负责人", "负责人联系电话"
    ];
    let dataList = [];
    let maxTeamNumber = 0;//最大团队数
    let maxHatching = 0;//最大场地数
    dataBaseList.forEach( info => {
        /**截取需要的返回字段 */
        let changeData:any = extractData(splitResultConfig.FuHuaQiBaseListConfig, info, true);

        const EnterpriseTotal = enterpriseMap[info.uscc] || 0; //企业总数
        let numOfEntities = EnterpriseTotal - (info.virtualEnterpriseNum || 0 ); //实体企业
        changeData.enterpriseTotal = EnterpriseTotal;
        changeData.numOfEntities = numOfEntities < 0 ? 0 : numOfEntities;

        /**枚举转换 */
        changeData.lv = changeEnumValue(configEnum.FUHUAQILV, changeData.lv);
        changeData.industry = changeEnumValue(configEnum.FUHUAINDUSTRY, changeData.industry);
        changeData.institutionalNature = changeEnumValue(configEnum.INSTITUTIONALNATURE, changeData.institutionalNature);
        changeData.operationModel = changeEnumValue(configEnum.OPERATIONMODEL, changeData.operationModel);
        changeData.foundingTeamType = changeEnumValue(configEnum.TEAM, changeData.foundingTeamType) || "无";

        /**算出最大的创始团队和场地 用于计算x轴下标 */
        if (changeData.foundingTeam && changeData.foundingTeam.length ) {
            maxTeamNumber = Math.max(changeData.foundingTeam.length, maxTeamNumber);
        }
        if (changeData.hatchingGround && changeData.hatchingGround.length ) {
            maxHatching = Math.max(changeData.hatchingGround.length, maxHatching);
        }

        /**生成除了创始团队和场地之外的 小数组 */
        let subList = [];
        keyList.forEach(subInfo => {
            subList.push(info[subInfo] || '');
        });

        dataList.push(subList);
    });

    /**根据最大的创始团队数量 生成标题 */
    let teamKeyList = ["type", "memberName", "memberSex", "memberAge", "memberDEGREE", "memberSchool", "des"];
    let teamTitleList = ["人才类型", "成员姓名", "成员性别", "成员年龄", "成员最高学历", "毕业学校", "履历描述"];
    for (let i = 0; i < maxTeamNumber; i++) {//平铺的插入数据
        let teamNum = i+1;
        teamTitleList.forEach(title => {
            titleList.push(`成员${teamNum}-${title}`);
        });
    }

    /**根据最大的场地数量 生成标题 */
    let hatchingKeyList = [ "personInChargeAdd", "siteAcreage", "leasePrice" ];
    let hatchingTitleList = ["孵化场地地址", "孵化场地面积", "孵化场地出租单价" ];
    for (let i = 0; i < maxHatching; i++) {
        let hatchingNumber = i+1;
        hatchingTitleList.forEach(title => {
            titleList.push(`场地${hatchingNumber}-${title}`);
        });
    }

    /**创始团队初始插入数据的x轴下标 = titleList的长度
     * 场地初始插入数据的x轴下标 = titleList的长度+(创始团队长度*标题个数)
     * 按照这个规则依次插入数据即可
    */
    dataBaseList.forEach((info, index) => {
        if (info.foundingTeam && info.foundingTeam.length ) {
            info.foundingTeam.forEach(teamInfo => {
                
                teamInfo.type = changeEnumValue(configEnum.TEAM, teamInfo.type);
                teamInfo.memberSex = changeEnumValue(configEnum.SEX, teamInfo.memberSex);
                teamInfo.memberDEGREE = changeEnumValue(configEnum.DEGREE, teamInfo.memberDEGREE);
                teamKeyList.forEach( teamKey => {
                    dataList[index].push(teamInfo[teamKey]);
                });
            });
        }
        if (info.hatchingGround && info.hatchingGround.length ) {
            
            info.hatchingGround.forEach((hatchingInfo, hatchingIndex) => {1
                hatchingKeyList.forEach( (hatchingKey, hatchingKeyIndex) => {
                    let addIndex = teamKeyList.length +  ( (hatchingIndex * 3 ) + (hatchingKeyIndex) )
                    dataList[index][addIndex] = hatchingInfo[hatchingKey];
                });
            });
        }
    });

    /**最后将标题作数组为二维数组的第一个子数组 */
    dataList.unshift(titleList)
    return dataList;
}

